#include <malloc.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <sys/types.h>
#include <stdlib.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#ifdef _WIN32
#include "ext/standard/info.h"
#endif

#include "dmydate.h"

#ifndef LONG
typedef long LONG;
#endif

#ifdef _WIN32
#define UTF8_STR(x)	EnsureUTF8(x, strlen(x))
inline void EnsureUTF8(char *pStr, int len)
{
	wchar_t *UTF8Str = (wchar_t*)malloc(len * sizeof(wchar_t));

	MultiByteToWideChar( CP_ACP, 0, pStr, -1, UTF8Str, len);
	WideCharToMultiByte(CP_UTF8, 0, UTF8Str, -1, pStr, len, NULL, NULL);
	
	free(UTF8Str);
}
#else
#define UTF8_STR(x)	x
inline void EnsureUTF8(char *pStr, int len){}
#endif

static function_entry dmyDate_functions[] = {
	PHP_FE(dmyDate_version, NULL)
	PHP_FE(dmyDiff_D, NULL)
	PHP_FE(dmyDiff_M, NULL)
	PHP_FE(dmyDiff_Y, NULL)
	{NULL, NULL, NULL}
};


zend_module_entry dmyDate_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	PHP_DMYDATE_WORLD_EXTNAME,
	dmyDate_functions,
	NULL,
	NULL,
	NULL,
	NULL,
	PHP_MINFO(dmyDate),	//    NULL,
#if ZEND_MODULE_API_NO >= 20010901
	PHP_DMYDATE_WORLD_VERSION,
#endif
	STANDARD_MODULE_PROPERTIES
};


#ifdef COMPILE_DL_DMYDATE
ZEND_GET_MODULE(dmyDate)
#endif

PHP_MINFO_FUNCTION(dmyDate)
{
	php_info_print_table_start();
	php_info_print_table_row(2, "dmyDiff",		"Dates Difference in Days, Months, Years");
	php_info_print_table_row(2, "dmyDiff_D",	"Show Date Difference in DMY - Days");
	php_info_print_table_row(2, "dmyDiff_M",	"Show Date Difference in DMY - Months");
	php_info_print_table_row(2, "dmyDiff_Y",	"Show Date Difference in DMY - Years");
	php_info_print_table_end();
}

PHP_FUNCTION(dmyDate_version)
{
	RETURN_STRING(PHP_DMYDATE_WORLD_VERSION, 1);
}

char g_tmpStr[1024];

PHP_FUNCTION(dmyDiff_D)
{
	time_t startDate, endDate, diffDate;
	struct tm dtS, dtE, dt;
	struct tm *res;
	LONG diff = 0;

	if (ZEND_NUM_ARGS() != 2)
    		WRONG_PARAM_COUNT;
     
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &startDate, &endDate) == FAILURE) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "ΔΕΝ ΕΧΟΥΝ ΔΩΘΕΙ ΕΓΚΥΡΕΣ ΗΜΕΡΟΜΗΝΙΕΣ");
		RETURN_FALSE;
	}

	if (endDate < startDate) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "ΔΕΝ ΜΠΟΡΕΙ ΝΑ ΥΠΟΛΟΓΙΣΤΕΙ Η ΔΙΑΦΟΡΑ");
		RETURN_FALSE;
	}

	res = php_localtime_r(&startDate, &dtS);
	res = php_localtime_r(&endDate, &dtE);

	sprintf(g_tmpStr, "startDate=%04d-%02d-%02d", dtS.tm_year+1900, dtS.tm_mon+1, dtS.tm_mday);
	sprintf(g_tmpStr, "endDate  =%04d-%02d-%02d", dtE.tm_year+1900, dtE.tm_mon+1, dtE.tm_mday);

	if (dtE.tm_mday < dtS.tm_mday)
	{
		if (dtS.tm_mon < 11) {
			dt.tm_year = dtS.tm_year;
			dt.tm_mon  = dtS.tm_mon + 1;
			dt.tm_mday = 1;
			dt.tm_hour = 0;
			dt.tm_min = 0;
			dt.tm_sec = 1;
			dt.tm_isdst = -1;
		} else {
			dt.tm_year = dtS.tm_year+1;
			dt.tm_mon  = 0;
			dt.tm_mday = 1;
			dt.tm_hour = 0;
			dt.tm_min = 0;
			dt.tm_sec = 1;
			dt.tm_isdst = -1;
		}
		diffDate = mktime(&dt);
		diffDate = (time_t)difftime(diffDate, startDate);
		res = php_localtime_r(&diffDate, &dt);
		//memcpy(&dt, localtime(&diffDate), sizeof(struct tm));

		diff = dt.tm_mday - 1 + dtE.tm_mday - 1;
	}
	else
	{
		diff = dtE.tm_mday - dtS.tm_mday;
	}

	RETURN_LONG(diff);
}

PHP_FUNCTION(dmyDiff_M)
{
	time_t startDate, endDate;
	struct tm dtS, dtE;
	struct tm *res;
	LONG diff = 0;

	if (ZEND_NUM_ARGS() != 2)
    		WRONG_PARAM_COUNT;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &startDate, &endDate) == FAILURE) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "ΔΕΝ ΕΧΟΥΝ ΔΩΘΕΙ ΕΓΚΥΡΕΣ ΗΜΕΡΟΜΗΝΙΕΣ");
		RETURN_FALSE;
	}

	if (endDate < startDate) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "ΔΕΝ ΜΠΟΡΕΙ ΝΑ ΥΠΟΛΟΓΙΣΤΕΙ Η ΔΙΑΦΟΡΑ");
		RETURN_FALSE;
	}

	res = php_localtime_r(&startDate, &dtS);
	res = php_localtime_r(&endDate, &dtE);

	diff = dtE.tm_mon - dtS.tm_mon;

	if (dtE.tm_mday < dtS.tm_mday)
		--diff;
	if (diff < 0)
		diff += 12;

	RETURN_LONG(diff);
}

PHP_FUNCTION(dmyDiff_Y)
{
	time_t startDate, endDate;
	struct tm dtS, dtE;
	struct tm *res;
	LONG diff = 0;

	if (ZEND_NUM_ARGS() != 2)
    		WRONG_PARAM_COUNT;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll", &startDate, &endDate) == FAILURE) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "ΔΕΝ ΕΧΟΥΝ ΔΩΘΕΙ ΕΓΚΥΡΕΣ ΗΜΕΡΟΜΗΝΙΕΣ");
		RETURN_FALSE;
	}

	if (endDate < startDate) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "ΔΕΝ ΜΠΟΡΕΙ ΝΑ ΥΠΟΛΟΓΙΣΤΕΙ Η ΔΙΑΦΟΡΑ");
		RETURN_FALSE;
	}

	res = php_localtime_r(&startDate, &dtS);
	res = php_localtime_r(&endDate, &dtE);

	diff = dtE.tm_year - dtS.tm_year;
	if (diff > 0)
	{
		if (dtE.tm_mon < dtS.tm_mon)
			--diff;
		else if ((dtE.tm_mon == dtS.tm_mon) && (dtE.tm_mday < dtS.tm_mday))
			--diff;
	}

	RETURN_LONG(diff);
}
